@@ -37,7 +37,7 @@ __author__ = "François Orieux"
 __copyright__ = "2021, François Orieux <francois.orieux@universite-paris-saclay.fr>"
 __credits__ = ["François Orieux"]
 __license__ = "GPL-3.0-or-later"
-__version__ = "0.9.0"
+__version__ = "0.12.1"
 __maintainer__ = "François Orieux"
 __email__ = "francois.orieux@universite-paris-saclay.fr"
 __status__ = "beta"
@@ -68,7 +68,7 @@ __all__ = [
 ]
 
 
-class OptimizeResult(dict):
+class OptimizeResult(dict):  # pylint: disable=too-many-instance-attributes
     """Represents the optimization result.
 
     x: array
@@ -83,7 +83,7 @@ class OptimizeResult(dict):
     nit: int
         Number of iterations performed by the optimizer.
     diff: list of float
-        The value of ||x^(k+1) - x^(k)||² at each iteration
+        The value of ||x_{k+1} - x_{k}||² at each iteration
     time: list of float
         The time at each iteration, starting at 0, in seconds.
     fun: float
@@ -120,8 +120,7 @@ class OptimizeResult(dict):
         """Last objective value"""
         if len(self.objv_val) != 0:
             return self.objv_val[-1]
-        else:
-            return None
+        return None
 
     @fun.setter
     def fun(self, value):
@@ -136,7 +135,7 @@ class OptimizeResult(dict):
 
     def __setattr__(self, name, value):
         if name == "fun":
-            return self["objv_val"].append(value)
+            self["objv_val"].append(value)
         self[name] = value
 
     def __delattr__(self, name):
@@ -147,9 +146,9 @@ class OptimizeResult(dict):
 
     def __repr__(self):
         if self.keys():
-            m = max(map(len, list(self.keys()))) + 1
+            rjust = max(map(len, list(self.keys()))) + 1
             return "\n".join(
-                [k.rjust(m) + ": " + str(v) for k, v in sorted(self.items())]
+                [k.rjust(rjust) + ": " + str(v) for k, v in sorted(self.items())]
             )
         return self.__class__.__name__ + "()"
 
@@ -157,18 +156,18 @@ class OptimizeResult(dict):
         return list(self.keys())
 
 
-def mmmg(
+def mmmg(  # pylint: disable=too-many-locals
     objv_list: Sequence["BaseObjective"],
-    x0: array,
+    x0: array,  # pylint: disable=invalid-name
     tol: float = 1e-4,
     max_iter: int = 500,
     precond: Optional[Callable[[array], array]] = None,
     callback: Optional[Callable[[OptimizeResult], None]] = None,
     calc_fun: bool = False,
 ) -> OptimizeResult:
-    r"""The Majorize-Minimize Memory Gradient (`3mg`) algorithm.
+    r"""The Majorize-Minimize Memory Gradient (`3MG`) algorithm.
 
-    The `mmmg` (`3mg`) algorithm is a subspace memory-gradient optimization
+    The `mmmg` (`3MG`) algorithm is a subspace memory-gradient optimization
     algorithm with an explicit step formula based on Majorize-Minimize Quadratic
     approach [2]_.
 
@@ -176,7 +175,7 @@ def mmmg(
     ----------
     objv_list : list of `BaseObjective`
         A list of :class:`BaseObjective` objects that each represents
-        a `μ ψ(V·x - ω)`. The objectives are summed.
+        a `μ ψ(Vx - ω)`. The objectives are summed.
     x0 : array
         The initial point.
     tol : float, optional
@@ -185,8 +184,8 @@ def mmmg(
     max_iter : int, optional
         The maximum number of iterations.
     precond : callable, optional
-        A callable that must implement a preconditioner, that is `P·x`. Must
-        be a callable with a unique input parameter `x` and unique output.
+        A callable that must implement a preconditioner, that is `Px`. Must be a
+        callable with a unique input parameter `x` and unique output like `x`.
     callback : callable, optional
         A function that receive the `OptimizeResult` at the end of each
         iteration.
@@ -204,6 +203,7 @@ def mmmg(
        Strategy for Subspace Optimization Applied to Image Restoration,” IEEE
        Trans. on Image Process., vol. 20, no. 6, pp. 1517–1528, Jun. 2011, doi:
        10.1109/TIP.2010.2103083.
+
     """
     if precond is None:
         precond = lambda x: x
@@ -220,7 +220,7 @@ def mmmg(
     # statement.
     move = np.zeros_like(res.x)
     op_directions = [
-        np.tile(vect(objv.operator, move, x0.shape), 2) for objv in objv_list
+        np.tile(vect_call(objv.operator, move, x0.shape), 2) for objv in objv_list
     ]
     step = np.ones((2, 1))
 
@@ -228,7 +228,7 @@ def mmmg(
 
     for iteration in range(max_iter):
         # Vectorized gradient
-        grad = _gradient(objv_list, res.x, x0.shape)
+        grad = vectgradient(objv_list, res.x, x0.shape)
         res.grad_norm.append(la.norm(grad))
         res.jac = grad.reshape(x0.shape)
         res.fun = lastgv(objv_list)
@@ -240,12 +240,12 @@ def mmmg(
             break
 
         # Memory gradient directions
-        new_dir = vect(precond, grad, x0.shape)
+        new_dir = vect_call(precond, grad, x0.shape)
         directions = np.c_[-new_dir, move]
 
         # Step by Majorize-Minimize
         op_directions = [
-            np.c_[vect(objv.operator, new_dir, x0.shape), i_op_dir @ step]
+            np.c_[vect_call(objv.operator, new_dir, x0.shape), i_op_dir @ step]
             for objv, i_op_dir in zip(objv_list, op_directions)
         ]
         step = -la.pinv(
@@ -283,9 +283,9 @@ def mmmg(
     return res
 
 
-def mmcg(
+def mmcg(  # pylint: disable=too-many-locals
     objv_list: Sequence["BaseObjective"],
-    x0: array,
+    x0: array,  # pylint: disable=invalid-name
     tol: float = 1e-4,
     max_iter: int = 500,
     precond: Optional[Callable[[array], array]] = None,
@@ -302,7 +302,7 @@ def mmcg(
     ----------
     objv_list : list of `BaseObjective`
         A list of :class:`BaseObjective` objects that each represents
-        a `μ ψ(V·x - ω)`. The objectives are summed.
+        a `μ ψ(Vx - ω)`. The objectives are summed.
     x0 : ndarray
         The initial point.
     tol : float, optional
@@ -311,8 +311,8 @@ def mmcg(
     max_iter : int, optional
         The maximum number of iterations.
     precond : callable, optional
-        A callable that must implement a preconditioner, that is `P·x`. Must
-        be a callable with a unique input parameter `x` and unique output.
+        A callable that must implement a preconditioner, that is `Px`. Must be a
+        callable with a unique input parameter `x` and unique output like `x`.
     callback : callable, optional
         A function that receive the `OptimizeResult` at the end of each
         iteration.
@@ -339,8 +339,9 @@ def mmcg(
 
     res.x = x0.copy().reshape((-1, 1))
 
-    residual = -_gradient(objv_list, res.x, x0.shape)
-    sec = vect(precond, residual, x0.shape)
+    # pylint: disable=invalid-unary-operand-type
+    residual = -vectgradient(objv_list, res.x, x0.shape)
+    sec = vect_call(precond, residual, x0.shape)
     direction = sec
     delta = residual.T @ direction
 
@@ -353,7 +354,9 @@ def mmcg(
             break
 
         # update
-        op_direction = [vect(objv.operator, direction, x0.shape) for objv in objv_list]
+        op_direction = [
+            vect_call(objv.operator, direction, x0.shape) for objv in objv_list
+        ]
 
         step = direction.T @ residual
         step = step / sum(
@@ -367,14 +370,14 @@ def mmcg(
         res.time.append(time.time())
 
         # Gradient
-        residual = -_gradient(objv_list, res.x, x0.shape)
+        residual = -vectgradient(objv_list, res.x, x0.shape)
         res.jac = -residual.reshape(x0.shape)
         res.fun = lastgv(objv_list)
 
         # Conjugate direction. No reset is done, see Shewchuck.
         delta_old = delta
         delta_mid = residual.T @ sec
-        sec = vect(precond, residual, x0.shape)
+        sec = vect_call(precond, residual, x0.shape)
         delta = residual.T @ sec
         if (delta - delta_mid) / delta_old >= 0:
             direction = sec + (delta - delta_mid) / delta_old * direction
@@ -401,9 +404,9 @@ def mmcg(
     return res
 
 
-def lcg(
+def lcg(  # pylint: disable=too-many-locals
     objv_list: Sequence["QuadObjective"],
-    x0: array,
+    x0: array,  # pylint: disable=invalid-name
     tol: float = 1e-4,
     max_iter: int = 500,
     precond: Optional[Callable[[array], array]] = None,
@@ -418,12 +421,12 @@ def lcg(
     ----------
     objv_list : list of `QuadObjective`
         A list of :class:`QuadObjective` objects that each represents
-        a `½ μ ||V·x - ω||²`. The objectives are summed.
+        a `½ μ ||Vx - ω||²_B`. The objectives are summed.
     x0 : ndarray
         The initial point.
     precond : callable, optional
-        A callable that must implement a preconditioner, that is `P·x`. Must
-        be a callable with a unique input parameter `x` and unique output.
+        A callable that must implement a preconditioner, that is `Px`. Must be a
+        callable with a unique input parameter `x` and unique output like `x`.
     tol : float, optional
         The stopping tolerance. The algorithm is stopped when the gradient norm
         is inferior to `x0.size * tol`.
@@ -448,18 +451,20 @@ def lcg(
 
     res.x = x0.copy().reshape((-1, 1))
 
-    second_term = np.reshape(reduce(iadd, (c.hdata_t for c in objv_list)), (-1, 1))
+    second_term = np.reshape(
+        reduce(iadd, (qobjv.VtB_data for qobjv in objv_list)), (-1, 1)
+    )
     constant = reduce(iadd, (c.constant for c in objv_list))
 
     def hessian(arr):
-        return reduce(iadd, (vect(c.hessp, arr, x0.shape) for c in objv_list))
+        return reduce(iadd, (vect_call(c.hessp, arr, x0.shape) for c in objv_list))
 
     def value_residual(arr, residual):
         return (np.sum(arr * (-second_term - residual)) + constant) / 2
 
     # Gradient at current x0
     residual = second_term - hessian(res.x)
-    direction = vect(precond, residual, x0.shape)
+    direction = vect_call(precond, residual, x0.shape)
 
     res.grad_norm.append(np.sum(np.real(np.conj(residual) * direction)))
     res.time.append(time.time())
@@ -484,7 +489,7 @@ def lcg(
         res.jac = -residual.reshape(x0.shape)
 
         # Conjugate direction with preconditionner
-        secant = vect(precond, residual, x0.shape)
+        secant = vect_call(precond, residual, x0.shape)
         res.grad_norm.append(np.sum(np.real(np.conj(residual) * secant)))
         direction = secant + (res.grad_norm[-1] / res.grad_norm[-2]) * direction
 
@@ -515,12 +520,16 @@ def lcg(
     return res
 
 
+#%% \
+# Utilities
+
 # Vectorized call
-def vect(func: Callable[[array], array], point: array, shape: Tuple) -> array:
+def vect_call(func: Callable[[array], array], point: array, shape: Tuple) -> array:
     """Call func with point reshaped as shape and return vectorized output"""
     return np.reshape(func(np.reshape(point, shape)), (-1, 1))
 
 
+# Not used in the module, only provided for user convenience
 def vectorize(shape: Tuple) -> Callable:
     """Return a decorator to vectorize input and output given `shape`"""
 
@@ -537,19 +546,98 @@ def vectorize(shape: Tuple) -> Callable:
 
 
 # Vectorized gradient
-def _gradient(
+def vectgradient(
     objv_list: Sequence["BaseObjective"], point: array, shape: Tuple
 ) -> array:
-    """Compute sum of gradient with vectorized parameters and return"""
-    # The use of reduce and iadd do an more efficient numpy inplace sum
-    return reduce(iadd, (vect(o.gradient, point, shape) for o in objv_list))
+    """Compute sum of gradient with vectorized parameters and return."""
+    # The use of reduce and iadd do a more efficient numpy inplace sum
+    return reduce(iadd, (vect_call(objv.gradient, point, shape) for objv in objv_list))
 
 
-def lastgv(objv_list: Sequence["BaseObjective"]):
+def lastgv(objv_list: Sequence["BaseObjective"]) -> float:
     """Return the value of objective computed after gradient evaluation"""
     return sum(objv.lastgv for objv in objv_list)
 
 
+class Stacked:
+    """A wrapper for list of array feature.
+
+    Interface for operator callable like
+
+         |V1|   y1
+    Vx = |: | = :
+         |Vq|   yq
+
+    associated with a list of output y = [y1, ..., yq].
+
+    If output shapes of V_i are shape_i, the `Stacked` operator return a
+    vectorized array of shape (M, 1) = (sum(prod(shape_i) for all i), 1).
+
+    Two methods are implemented.
+
+    - `operator` takes an array `x` in shape (N, ..., P), apply the callable
+      `V`, and return a the concatenation of [y1, ..., yq] in a array `y` of
+      shape (M, 1).
+
+    - `adjoint` takes an array `y` in shape (M, 1), split in a list `[y1, ...
+      yq]` and apply the callable `Vᵀ`.
+
+    Attributes
+    ----------
+    - shapes : list of tuple
+      The list of output shape
+    - data: array
+      The vectorized data.
+
+    """
+
+    def __init__(
+        self,
+        operator: Callable[[array], Sequence[array]],
+        adjoint: Callable[[Sequence[array]], array],
+        shapes: List[Tuple[int]],
+    ):
+        """A wrapper for list of array feature.
+
+        Parameters
+        ----------
+        - operator: callable
+          Apply V
+        - adjoint: callable
+          Apply Vᵀ
+        - shapes: list of shape
+          Shape of each output of Vx.
+        """
+        self.shapes = shapes
+        self._operator = operator
+        self._adjoint = adjoint
+        self._idx = np.cumsum([0] + [np.prod(shape) for shape in shapes])
+
+    @staticmethod
+    def list2vec(arr_list: Sequence[array]) -> array:
+        """Vectorize a list of array."""
+        return np.vstack([arr.reshape((-1, 1)) for arr in arr_list])
+
+    def vec2list(self, arr: array) -> List[array]:
+        """De-vectorize to a list of array."""
+        return [
+            np.reshape(arr[self._idx[i] : self._idx[i + 1]], shape)
+            for i, shape in enumerate(self.shapes)
+        ]
+
+    def operator(self, point: array) -> array:
+        """Return `Vx` as array."""
+        return self.list2vec(self._operator(point))
+
+    def adjoint(self, point: array) -> array:
+        """Return `Vᵀx` from array."""
+        return self._adjoint(self.vec2list(point))
+
+
+#%% \
+# Objectives
+
+
 class BaseObjective(abc.ABC):
     r"""An abstract base class for objective function
 
@@ -565,21 +653,24 @@ class BaseObjective(abc.ABC):
         in `lastgv` attribute (False by default).
     name: str
         The name of the objective.
+    hyper: float
+        The hyperparameter value `μ`.
     lastv: float
         The last evaluated value of the objective (0 by default).
     lastgv: float
         The value of objective obtained during gradient computation (0 by default).
     """
 
-    def __init__(self, name=""):
+    def __init__(self, hyper=1, name=""):
         self.lastgv = 0
         self.lastv = 0
         self.calc_fun = False
+        self.hyper = hyper
         self.name = name
 
     @abc.abstractmethod
     def operator(self, point: array) -> array:
-        """Compute the output of `V·x`."""
+        """Compute the output of `Vx`."""
         return NotImplemented
 
     @abc.abstractmethod
@@ -615,6 +706,9 @@ class BaseObjective(abc.ABC):
         """
         return NotImplemented
 
+    def __call__(self, point: array) -> float:
+        return self.value(point)
+
     def __add__(self, objv: "BaseObjective"):
         return MixedObjective([self, objv])
 
@@ -681,9 +775,7 @@ class MixedObjective(collections.abc.MutableSequence):
             self._objv_list.extend(objv._objv_list)
         else:
             raise TypeError(
-                "unsupported operand type(s) for +: 'MixedObjective' and {}".format(
-                    type(objv)
-                )
+                f"unsupported operand type(s) for +: 'MixedObjective' and {type(objv)}"
             )
         return self
 
@@ -692,9 +784,7 @@ class MixedObjective(collections.abc.MutableSequence):
             self._objv_list.append(objv)
         else:
             raise TypeError(
-                "unsupported operand type(s) for +: 'MixedObjective' and {}".format(
-                    type(objv)
-                )
+                f"unsupported operand type(s) for +: 'MixedObjective' and {type(objv)}"
             )
         return self
 
@@ -707,6 +797,7 @@ class Objective(BaseObjective):
 
     with :math:`\Psi(u) = \sum_i \varphi(u_i)`.
 
+    The instance attributs are:
 
     data : array
         The `data` array, or the vectorized list of array given at init.
@@ -725,78 +816,55 @@ class Objective(BaseObjective):
         hyper: float = 1,
         name: str = "",
     ):
-        """A objective function `μ ψ(V·x - ω)`.
+        """A objective function `μ ψ(Vx - ω)`.
 
         Parameters
         ----------
         operator: callable
-            A callable that compute the output `V·x`.
+            A callable that compute the output `Vx`.
         adjoint: callable
-            A callable that compute `Vᵀ·e`.
+            A callable that compute `Vᵀe`.
         loss: Loss
             The loss `φ`.
         data: array or list of array, optional
             The data vector `ω`.
         hyper: float, optional
             The hyperparameter `μ`.
-        name: str
+        name: str, optional
             The name of the objective.
 
         Notes
         -----
-        For implementation issue, `operator` and `adjoint` are wrapped by
-        methods of same name.
-
         If `data` is a list of array, `operator` must return a similar list with
         arrays of same shape, and `adjoint` must accept a similar list also.
 
         In that case, however, and for algorithm purpose, everything is
-        internally stacked as a column vector and values are therefore copied.
-        This is not efficient but flexible. Users are encouraged to do the
-        vectorization themselves and not use the "list of array" feature.
-        """
-        super().__init__(name=name)
-        self._operator = operator
-        self._adjoint = adjoint
+        internally stacked as a column vector and values are therefore copied,
+        by using a `Stacked` object. This is not efficient but flexible. Users
+        are encouraged to do the vectorization themselves and use this "list of
+        array" feature.
 
+        """
+        super().__init__(hyper=hyper, name=name)
         if isinstance(data, list):
-            self._shape = [arr.shape for arr in data]
-            self._idx = np.cumsum([0] + [arr.size for arr in data])
-            self.data = self._list2vec(data)
+            stacked = Stacked(operator, adjoint, [dat.shape for dat in data])
+            self._op: Callable[[array], array] = stacked.operator
+            self._adj: Callable[[array], array] = stacked.adjoint
+            self.data = stacked.list2vec(data)
         else:
+            self._op = operator
+            self._adj = adjoint
             self.data = 0 if data is None else data
 
-        self.hyper = hyper
         self.loss = loss
 
-    @staticmethod
-    def _list2vec(arr_list: Sequence[array]) -> array:  #  pylint: disable=no-self-use
-        """Vectorize a list of array."""
-        return np.vstack([arr.reshape((-1, 1)) for arr in arr_list])
-
-    def _vec2list(self, arr: array) -> List[array]:
-        """De-vectorize to a list of array."""
-        return [
-            np.reshape(arr[self._idx[i] : self._idx[i + 1]], shape)
-            for i, shape in enumerate(self._shape)
-        ]
-
     def operator(self, point: array) -> array:
-        """Return `V·x`."""
-        if hasattr(self, "_shape"):
-            return self._list2vec(self._operator(point))
-        return self._operator(point)
-
-    def adjoint(self, point: array) -> array:
-        """Return `Vᵀ·x`."""
-        if hasattr(self, "_shape"):
-            return self._adjoint(self._vec2list(point))
-        return self._adjoint(point)
+        return self._op(point)
 
     def value(self, point: array) -> float:
         """The value of the objective function at given point
 
-        Return `μ ψ(V·x - ω)`.
+        Return `μ ψ(Vx - ω)`.
         """
         self.lastv = self.hyper * np.sum(self.loss(self.operator(point) - self.data))
         return self.lastv
@@ -804,12 +872,12 @@ class Objective(BaseObjective):
     def gradient(self, point: array) -> array:
         """The gradient and value at given point
 
-        Return `μ Vᵀ·φ'(V·x - ω)`.
+        Return `μ Vᵀφ'(Vx - ω)`.
         """
         residual = self.operator(point) - self.data
         if self.calc_fun:
             self.lastgv = self.hyper * np.sum(self.loss(residual))
-        return self.hyper * self.adjoint(self.loss.gradient(residual))
+        return self.hyper * self._adj(self.loss.gradient(residual))
 
     def norm_mat_major(self, vecs: array, point: array) -> array:
         matrix = np.real(
@@ -820,19 +888,24 @@ class Objective(BaseObjective):
     def gr_coeffs(self, point: array) -> array:
         """The Geman & Reynolds coefficients at given point
 
-        Given `x` return `φ'(V·x - ω) / (V·x - ω)`
+        Given `x` return `φ'(Vx - ω) / (Vx - ω)`.
         """
         obj = self.operator(point) - self.data
         return self.loss.gr_coeffs(obj)
 
-    def __call__(self, point: array) -> float:
-        return self.value(point)
+    def gy_coeffs(self, point: array) -> array:
+        """The Geman & Yang coefficients at given point
+
+        Given `x` return `Vx - φ'(Vx - ω)`.
+        """
+        aux = self.operator(point)
+        return aux - self.loss.gradient(aux - self.data)
 
     def __repr__(self):
         return f"{self.name} ({type(self).__name__}, loss: {type(self.loss).__name__}), hyper {self.hyper}, last eval {self.lastv}"
 
 
-class QuadObjective(Objective):
+class QuadObjective(BaseObjective):
     r"""A quadratic objective function
 
     .. math::
@@ -845,12 +918,14 @@ class QuadObjective(Objective):
         \end{aligned}
         \end{equation}
 
-    data : array
-        The `data` array, or the vectorized list of array given at init.
+    The instance attributes are:
+
     hyper : float
         The hyperparameter value `μ`.
-    data_t : array
-        The retroprojected data `B·Vᵀ·ω`.
+    ht_data : array
+        The retroprojected data `μ VᵀBω`.
+    constant : float
+        The constant value `μ ωᵀBω`.
     """
 
     def __init__(  # pylint: disable=too-many-arguments
@@ -863,40 +938,53 @@ class QuadObjective(Objective):
         invcovp: Callable[[array], array] = None,
         name: str = "",
     ):
-        """A quadratic objective `½ μ ||V·x - ω||²_B`
+        """A quadratic objective `½ μ ||Vx - ω||²_B`
 
         Parameters
         ----------
         operator: callable
-            A callable that compute the output `V·x`.
+            A callable that compute the output `Vx`.
         adjoint: callable
-            A callable that compute `Vᵀ·e`.
+            A callable that compute `Vᵀe`.
         hessp: callable, optional
-            A callable that compute `Q·x` as `Q·x = VᵀV·x`
+            A callable that compute `Qx` as `Qx = VᵀBVx`. Must take a parameter
+            like `operator` and return a parameter like `adjoint` (`x`-like in
+            both case).
         data: array or list of array, optional
             The data vector `ω`.
         hyper: float, optional
             The hyperparameter `μ`.
         invcovp: callable, optional
-            A callable that apply the inverse covariance, or metric, `B`.
-            Equivalent to Identity if not provided.
-        name: str
+            A callable, that take a parameter like `adjoint` and return like
+            `operator` (`ω`-like in both case), that apply the inverse
+            covariance, or metric, `B=Σ⁻¹`. Equivalent to Identity if `None`.
+        name: str, optional
             The name of the objective.
 
         Notes
         -----
         The `hessp` (`Q`) callable is used for gradient computation as `∇ = μ
-        (Q·x - b)` where `b = B·Vᵀ·ω` instead of `∇ = μ Vᵀ·B·(V·x - ω)`. This is
-        optional and in some case this is more efficient.
+        (Qx - b)` where `b = VᵀBω` instead of `∇ = μ VᵀB(Vx - ω)`. This is
+        optional and in some case can be more efficient. Use it only in that case.
 
-        The variable `b = B·Vᵀ·ω` is computed at object creation.
+        The variable `b = VᵀBω` is computed at object initialisation.
 
         """
-        super().__init__(operator, adjoint, Square(), data=data, hyper=hyper, name=name)
+        super().__init__(hyper=hyper, name=name)
+
+        self._unvec_op = operator
+        if isinstance(data, list):
+            self._op: Callable = Stacked(
+                operator, adjoint, [dat.shape for dat in data]
+            ).operator
+        else:
+            self._op = operator
+        self._adj = adjoint
+        self._data = data
 
         if invcovp is None:
-            self.invcovp = lambda x: x
-        else:
+            self.invcovp = lambda x: x  # Identity
+        elif invcovp is not None:
             self.invcovp = invcovp
 
         if hessp is not None:
@@ -905,66 +993,98 @@ class QuadObjective(Objective):
             self.hessp = lambda x: hyper * adjoint(self.invcovp(operator(x)))
 
         if data is None:
-            self.hdata_t = 0
-            self.constant = 0  # c = μ ωᵀ·B·ω
+            # b = μ VᵀBω
+            self._vtb_data = 0
+            # c = μ ωᵀBω
+            self._cst = 0
         else:
-            self.hdata_t = hyper * self.invcovp(adjoint(data))
-            self.constant = hyper * np.sum(data * self.invcovp(data))  # c = μ ωᵀ·B·ω
+            # Lazy computation of second term b and constant c
+            self._vtb_data = None
+            self._cst = None
+
+    @property
+    def VtB_data(self):  # pylint: disable=invalid-name
+        """The second term b = μ ∑ᵢ VᵢᵀBᵢωᵢ"""
+        if self._vtb_data is None:
+            self._vtb_data = self.hyper * self._adj(self.invcovp(self._data))
+        return self._vtb_data
+
+    @property
+    def constant(self):
+        """The constant c = μ ∑_i ωᵢᵀBᵢωᵢ"""
+        if self._cst is None:
+            if isinstance(self._data, list):
+                self._cst = self.hyper * sum(
+                    np.real(np.sum(y * By))
+                    for y, By in zip(self._data, self.invcovp(self._data))
+                )
+            else:
+                self._cst = self.hyper * np.real(
+                    np.sum(self._data * self.invcovp(self._data))
+                )
+        return self._cst
+
+    def operator(self, point: array) -> array:
+        return self._op(point)
 
     def value(self, point: array) -> float:
         """The value of the objective function at given point
 
-        Return `½ μ ||V·x - ω||²_B`.
+        Return `½ μ ||Vx - ω||²_B`.
         """
-        diff = self.operator(point) - self.data
-        self.lastv = self.hyper * np.sum(diff * self.invcovp(diff)) / 2
+        # Compute J(x) = ½ (μ xVᵀBVx - 2 xᵀb + c), with b = μVᵀBω and c = μωᵀBω
+        Vx = self._unvec_op(point)  # pylint: disable=invalid-name
+        # xVᵀBVx
+        if isinstance(Vx, list):
+            self.lastv = self.hyper * sum(
+                np.sum(Vx_i * BVx_i) for Vx_i, BVx_i in zip(Vx, self.invcovp(Vx))
+            )
+        else:
+            self.lastv = self.hyper * np.sum(Vx * self.invcovp(Vx))
+        self.lastv = (
+            self.lastv - 2 * np.sum(point * self.VtB_data) + self.constant
+        ) / 2
         return self.lastv
 
     def gradient(self, point: array) -> array:
         """The gradient and value at given point
 
-        Return `∇ = μ (Q·x - b) = μ Vᵀ·B·(V·x - ω)`.
+        Return `∇ = μ (Qx - b) = μ VᵀB(Vx - ω)`.
 
         Notes
         -----
-        Objective value is computed with low overhead thanks to the relation
+        Objective value is always computed with low overhead thanks to the
+        relation
 
-        `J(x) = ½ (xᵀ·∇ - xᵀ·b + μ ωᵀ·B·ω)`
+        `J(x) = ½ (xᵀ∇ - xᵀb + μ ωᵀBω)`.
         """
-        Qx = self.hessp(point)
-        self.lastgv = self.value_hessp(point, Qx)
-        return self.hessp(point) - self.hdata_t
+        hessp = self.hessp(point)
+        self.lastgv = self.value_hessp(point, hessp)
+        return self.hessp(point) - self.VtB_data
 
     def value_hessp(self, point, hessp):
         """Return `J(x)` value at low cost given `x` and `q = Qx`
 
-        thanks to relation
+        thanks to the relation
 
-        `J(x) =  ½ (xᵀ·q - 2 xᵀ·b + μ ωᵀ·B·ω)`"""
+        `J(x) =  ½ (xᵀq - 2 xᵀb + μ ωᵀBω)`."""
         return (
             np.sum(np.reshape(point, (-1)) * np.reshape(hessp, (-1)))
-            - 2 * np.sum(np.reshape(point, (-1)) * np.reshape(self.hdata_t, (-1)))
+            - 2 * np.sum(np.reshape(point, (-1)) * np.reshape(self.VtB_data, (-1)))
             + self.constant
         ) / 2
 
     def value_residual(self, point, residual):
         """Return `J(x)` value at low cost given `x` and `r = b - Qx`
 
-        thanks to relation
+        thanks to the relation
 
-        `J(x) =  ½ (xᵀ·(-b - r) + μ ωᵀ·B·ω)`"""
-        return (np.sum(point * (-self.hdata_t - residual)) + self.constant) / 2
+        `J(x) =  ½ (xᵀ(-b - r) + μ ωᵀBω)`."""
+        return (np.sum(point * (-self.VtB_data - residual)) + self.constant) / 2
 
     def norm_mat_major(self, vecs: array, point: array) -> array:
         return np.real(np.conj(vecs.T) @ vecs)
 
-    def gr_coeffs(self, point: array) -> array:
-        """Return 1."""
-        return 1
-
-    def __call__(self, point: array) -> float:
-        return self.value(point)
-
 
 class Vmin(BaseObjective):
     r"""A minimum value objective function
@@ -993,14 +1113,13 @@ class Vmin(BaseObjective):
         name: str
             The name of the objective.
         """
-        super().__init__(name=name)
+        super().__init__(hyper=hyper, name=name)
         self.vmin = vmin
-        self.hyper = hyper
 
     def operator(self, point):
         return point[point <= self.vmin]
 
-    def value(self, point: array) -> array:
+    def value(self, point: array) -> float:
         """Return the value at current point."""
         return self.hyper * np.sum((point[point <= self.vmin] - self.vmin) ** 2) / 2
 
@@ -1019,7 +1138,7 @@ class Vmax(BaseObjective):
 
     .. math::
 
-        J(x) = \frac{1}{2} \mu \|P_{[M, +\infty[}(x) - m\|_2^2.
+        J(x) = \frac{1}{2} \mu \|P_{[M, +\infty[}(x) - M\|_2^2.
 
     vmax : float
         The maximum value `M`.
@@ -1041,14 +1160,13 @@ class Vmax(BaseObjective):
         name: str
             The name of the objective.
         """
-        super().__init__(name=name)
+        super().__init__(hyper=hyper, name=name)
         self.vmax = vmax
-        self.hyper = hyper
 
     def operator(self, point):
         return point[point >= self.vmax]
 
-    def value(self, point: array) -> array:
+    def value(self, point: array) -> float:
         """Return the value at current point."""
         return self.hyper * np.sum((point[point >= self.vmax] - self.vmax) ** 2) / 2
 
@@ -1062,6 +1180,10 @@ class Vmax(BaseObjective):
         return np.real(np.conj(vecs.T) @ vecs)
 
 
+#%% \
+# Objectives
+
+
 class Loss(abc.ABC):
     """An abstract base class for loss `φ`.
 
@@ -1108,6 +1230,10 @@ class Loss(abc.ABC):
         aux[idx] = self.gradient(point[idx]) / point[idx]
         return aux
 
+    def gy_coeffs(self, point: array) -> array:
+        """The Geman & Yang `x - φ'(x)` coefficients at given point."""
+        return point - self.gradient(point)
+
     def __call__(self, point: array) -> array:
         """The value at given point."""
         return self.value(point)
@@ -1126,7 +1252,7 @@ class Square(Loss):
         super().__init__(inf=1, convex=True, coercive=True)
 
     def value(self, point: array) -> array:
-        return point ** 2 / 2
+        return point**2 / 2
 
     def gradient(self, point: array) -> array:
         return point
@@ -1157,7 +1283,7 @@ class Huber(Loss):
     def value(self, point: array) -> array:
         return np.where(
             np.abs(point) <= self.delta,
-            point ** 2 / 2,
+            point**2 / 2,
             self.delta * (np.abs(point) - self.delta / 2),
         )
 
@@ -1193,10 +1319,10 @@ class Hyperbolic(Loss):
         self.delta = delta
 
     def value(self, point: array) -> array:
-        return self.delta ** 2 * (np.sqrt(1 + (point ** 2) / (self.delta ** 2)) - 1)
+        return self.delta**2 * (np.sqrt(1 + (point**2) / (self.delta**2)) - 1)
 
     def gradient(self, point: array) -> array:
-        return point / np.sqrt(1 + (point ** 2) / self.delta ** 2)
+        return point / np.sqrt(1 + (point**2) / self.delta**2)
 
     def __repr__(self):
         return f"""{type(self)}
@@ -1221,14 +1347,14 @@ class HebertLeahy(Loss):
 
     def __init__(self, delta: float):
         """The Hebert & Leahy loss."""
-        super().__init__(inf=2 / delta ** 2, convex=False, coercive=True)
+        super().__init__(inf=2 / delta**2, convex=False, coercive=True)
         self.delta = delta
 
     def value(self, point: array) -> array:
-        return np.log(1 + point ** 2 / self.delta ** 2)
+        return np.log(1 + point**2 / self.delta**2)
 
     def gradient(self, point: array) -> array:
-        return 2 * point / (self.delta ** 2 + point ** 2)
+        return 2 * point / (self.delta**2 + point**2)
 
     def __repr__(self):
         return f"""{type(self)}
@@ -1252,14 +1378,14 @@ class GemanMcClure(Loss):
 
     def __init__(self, delta: float):
         r"""The Geman & Mc Clure loss."""
-        super().__init__(1 / (delta ** 2), convex=False, coercive=False)
+        super().__init__(1 / (delta**2), convex=False, coercive=False)
         self.delta = delta
 
     def value(self, point: array) -> array:
-        return point ** 2 / (2 * self.delta ** 2 + point ** 2)
+        return point**2 / (2 * self.delta**2 + point**2)
 
     def gradient(self, point: array) -> array:
-        return 4 * point * self.delta ** 2 / (2 * self.delta ** 2 + point ** 2) ** 2
+        return 4 * point * self.delta**2 / (2 * self.delta**2 + point**2) ** 2
 
     def __repr__(self):
         return f"""{type(self)}
@@ -1283,14 +1409,14 @@ class TruncSquareApprox(Loss):
 
     def __init__(self, delta: array):
         """The truncated square approximation."""
-        super().__init__(inf=1 / (delta ** 2), convex=False, coercive=False)
+        super().__init__(inf=1 / (delta**2), convex=False, coercive=False)
         self.delta = delta
 
     def value(self, point: array) -> array:
-        return 1 - np.exp(-(point ** 2) / (2 * self.delta ** 2))
+        return 1 - np.exp(-(point**2) / (2 * self.delta**2))
 
     def gradient(self, point: array) -> array:
-        return point / (self.delta ** 2) * np.exp(-(point ** 2) / (2 * self.delta ** 2))
+        return point / (self.delta**2) * np.exp(-(point**2) / (2 * self.delta**2))
 
     def __repr__(self):
         return f"""{type(self)}
